/* TODO: IRQ probe */

#include <stdio.h>
#include <dos.h>
#include <conio.h>

#include "dev.h"
#include "cdu31a.h"
#include "irqprobe.h"


#define SONY_STATUS_REG  0
#define SONY_CMD_REG     0
#define SONY_PARAM_REG   1
#define SONY_RESULT_REG  1
#define SONY_WRITE_REG   2
#define SONY_CONTROL_REG 3


/* Control reg bits */
#define SONY_ATTN_CLR_BIT        0x01
#define SONY_RES_RDY_CLR_BIT     0x02
#define SONY_DATA_RDY_CLR_BIT    0x04
#define SONY_ATTN_INT_EN_BIT     0x08
#define SONY_RES_RDY_INT_EN_BIT  0x10
#define SONY_DATA_RDY_INT_EN_BIT 0x20
#define SONY_PARAM_CLR_BIT       0x40
#define SONY_DRIVE_RESET_BIT     0x80

/* Status reg */
#define SONY_ATTN_BIT            0x01
#define SONY_RES_RDY_BIT         0x02
#define SONY_DATA_RDY_BIT        0x04
#define SONY_ATTN_INT_ST_BIT     0x08
#define SONY_RES_RDY_INT_ST_BIT  0x10
#define SONY_DATA_RDY_INT_ST_BIT 0x20
#define SONY_DATA_REQUEST_BIT    0x40
#define SONY_BUSY_BIT            0x80


/* Commands to request or set drive control parameters and disc information */
#define SONY_REQ_DRIVE_CONFIG_CMD       0x00    /* Returns s_sony_drive_config */
#define SONY_REQ_DRIVE_MODE_CMD         0x01
#define SONY_REQ_DRIVE_PARAM_CMD        0x02
#define SONY_REQ_MECH_STATUS_CMD        0x03
#define SONY_REQ_AUDIO_STATUS_CMD       0x04
#define SONY_SET_DRIVE_PARAM_CMD        0x10
#define SONY_REQ_TOC_DATA_CMD           0x20    /* Returns s_sony_toc */
#define SONY_REQ_SUBCODE_ADDRESS_CMD    0x21    /* Returns s_sony_subcode */
#define SONY_REQ_UPC_EAN_CMD            0x22
#define SONY_REQ_ISRC_CMD               0x23
#define SONY_REQ_TOC_DATA_SPEC_CMD      0x24    /* Returns s_sony_session_toc */

/* Commands to request information from the drive */
#define SONY_READ_TOC_CMD               0x30    /* let the drive firmware grab the TOC */
#define SONY_SEEK_CMD                   0x31
#define SONY_READ_CMD                   0x32
#define SONY_READ_BLKERR_STAT_CMD       0x34
#define SONY_ABORT_CMD                  0x35
#define SONY_READ_TOC_SPEC_CMD          0x36

/* Commands to control audio */
#define SONY_AUDIO_PLAYBACK_CMD         0x40
#define SONY_AUDIO_STOP_CMD             0x41
#define SONY_AUDIO_SCAN_CMD             0x42

/* Miscellaneous control commands */
#define SONY_EJECT_CMD                  0x50
#define SONY_SPIN_UP_CMD                0x51
#define SONY_SPIN_DOWN_CMD              0x52

/* Diagnostic commands */
#define SONY_WRITE_BUFFER_CMD           0x60
#define SONY_READ_BUFFER_CMD            0x61
#define SONY_DIAGNOSTICS_CMD            0x62

/*
 * The following are command parameters for the set drive parameter command
 */
#define SONY_SD_DECODE_PARAM            0x00
#define SONY_SD_INTERFACE_PARAM         0x01
#define SONY_SD_BUFFERING_PARAM         0x02
#define SONY_SD_AUDIO_PARAM             0x03
#define SONY_SD_AUDIO_VOLUME            0x04
#define SONY_SD_MECH_CONTROL            0x05
#define SONY_SD_AUTO_SPIN_DOWN_TIME     0x06


struct s_sony_drive_config
{
   char vendor_id[8];
   char product_id[16];
   char product_rev_level[8];
   unsigned char hw_config[2];
};


static const char *load_mech[] =
     { "Caddy", "Tray", "Pop-up", "Unknown" };


int cdu31a_probe(int port)
{
 int timeout;
 int rc, buflen;
 int x;
 char vendor_id[9];
 char product_id[17];
 char product_rev[9];
 int config1,config2;
 isadev *dev;

 if (!ask_for_probe("CDU31A interface",port))
   return 0;


//  if (sony_pas_init)
 outp(0x9a01, 0xbc);
 outp(0x9a01, 0xe2);

 if (inp(port+SONY_STATUS_REG)==0xFF)
   return 0;

// printf("port=0x%x\n",port);

 //reset drive
 outp(port+SONY_CONTROL_REG, SONY_DRIVE_RESET_BIT);

 timeout = 0;
 while ( ((inp(port+ SONY_STATUS_REG) & SONY_ATTN_BIT) == 0) )
  {
   timeout++;
   delay(1);
   if (timeout > 1000)
     return 0;
  }

// printf("Sending CONFIG command\n");

 outp(port + SONY_CONTROL_REG, SONY_RES_RDY_CLR_BIT);
 delay(1);
 outp(port + SONY_CONTROL_REG, SONY_PARAM_CLR_BIT);
 delay(1);

 // not necessary
 // write_params(params, num_params);

 outp(port + SONY_CONTROL_REG, SONY_RES_RDY_INT_EN_BIT);
 outp(port + SONY_CMD_REG, SONY_REQ_DRIVE_CONFIG_CMD);


// printf("WB\n");
 timeout = 0;
 while ( ((inp(port+ SONY_STATUS_REG) & SONY_BUSY_BIT) != 0) )
  {
   timeout++;
   delay(1);
   if (timeout > 1000)
     return 0;
  }
// printf("WR\n");
 timeout = 0;
 while ( ((inp(port+ SONY_STATUS_REG) & SONY_RES_RDY_BIT) != 0) )
  {
   timeout++;
   delay(1);
   if (timeout > 1000)
     return 0;
  }

// printf("OK.\n");
 delay(1);

 rc = inp(port+SONY_RESULT_REG);
 delay(1);
 rc = inp(port+SONY_RESULT_REG);
// printf("RC=%u\n",rc);
 delay(1);
 buflen = inp(port+SONY_RESULT_REG);
// printf("len=%u\n",buflen);
 delay(1);

 if (buflen!=34)
   return 0;

 for (x=0;x<8;x++) {
   vendor_id[x] = inp(port+SONY_RESULT_REG);
   delay(1);
 }
 vendor_id[8] = 0;

 for (x=0;x<16;x++) {
   product_id[x] = inp(port+SONY_RESULT_REG);
   delay(1);
 }
 product_id[16] = 0;

 for (x=0;x<8;x++) {
   product_rev[x] = inp(port+SONY_RESULT_REG);
   delay(1);
 }
 product_rev[8] = 0;

 config1 = inp(port+SONY_RESULT_REG);
 delay(1);
 config2 = inp(port+SONY_RESULT_REG);
 delay(1);

 dev = alloc_isadev();
 setisadevname(dev,"CDU31A interface");
 setisadevport(dev,port);

// printf("Vendor ID:   \"%s\"\n",vendor_id);
 add_isadevproperty(dev,"Vendor",vendor_id);
// printf("Product ID:  \"%s\"\n",product_id);
 add_isadevproperty(dev,"Product",product_id);
// printf("Product rev: \"%s\"\n",product_rev);
 add_isadevproperty(dev,"Revison",product_rev);
// printf("Loading mech:%s\n",load_mech[config1&0x03]);
 add_isadevproperty(dev,"Loading",load_mech[config1&0x03]);
// printf("Speed:       %s\n",config1&0x10?"2x":"1x");
 add_isadevproperty(dev,"Speed",config1&0x10?"2x":"1x");
// printf("Led support: %s\n",config1&0x08?"Yes":"No");
 add_isadevproperty(dev,"LED support",config1&0x08?"Yes":"No");
// printf("Eject:       %s\n",config1&0x04?"Yes":"No");
 add_isadevproperty(dev,"Eject",config1&0x04?"Yes":"No");
// printf("Buffer size: %u\n",(config1&0xc0) >>6);
 add_isadevpropertyf(dev,"Buffer size","%u",(config1&0xc0) >>6);
// printf("Audio:       %s\n",config2&0x01?"Yes":"No");
 add_isadevproperty(dev,"Audio",config2&0x01?"Yes":"No");
// printf("Volume:      %s\n",config2&0x02?"Yes":"No");
 add_isadevproperty(dev,"Volume",config2&0x08?"Yes":"No");
// printf("Volume ctrl: %s\n",config2&0x04?"Yes":"No");
 add_isadevproperty(dev,"Volume ctrl",config1&0x04?"Yes":"No");

 addisadev(dev);

 return 0;
}