#include <malloc.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include "dev.h"
#include "lib/cons.h"

isadev *isa_device_list;

isadev * alloc_isadev (void)
{
 isadev *dev;

 dev = (isadev *)malloc(sizeof(isadev));
 if (!dev)
  return 0;

 dev->name = 0;
 dev->iobase = -1;
 dev->nirqs = 0;
 dev->properties = 0;

 return dev;
}

void addisadev(isadev *dev)
{
 dev->next = isa_device_list;
 isa_device_list = dev;
}


void setisadevname(isadev *dev,char *name)
{
 if (!dev)
  return;
 if (dev->name)
  free(dev->name);
 dev->name = strdup(name);
}

void setisadevport(isadev *dev,int port)
{
 if (!dev)
  return;
 dev->iobase = port;
}

void allocisadevirq(isadev *dev,int irq)
{
 if (!dev)
  return;
 if (dev->nirqs<MAX_IRQS-1)
  {
   dev->irqs[dev->nirqs] = irq;
   dev->nirqs++;
  }
}

void add_isadevproperty(isadev *dev,char *name,char *value)
{
 property *p;

 if (!dev)
  return;
 p = (property*)malloc(sizeof(property));
 if (!p)
  return;
 p->name = strdup(name);
 if (!p->name)
  {
   free(p);
   return;
  }
 p->value = strdup(value);
 if (!p->value)
  {
   free(p->name);
   free(p);
   return;
  }
 p->next = dev->properties;
 dev->properties = p;
}

void add_isadevpropertyf(isadev *dev,char *name,char *fmt,...)
{
 char buffer[80];

 va_list argptr;
 int cnt;

 va_start(argptr, fmt);
 vsprintf(buffer, fmt, argptr);         /* DANGEROUS: vsnprintf (!!) */
 va_end(argptr);

 add_isadevproperty(dev,name,buffer);
}


void init_dev()
{
 isa_device_list = 0;
}

void dump_devices()
{
 isadev *dev;
 property *p;
 int x;

 dev = isa_device_list;

 printf("\nOther probed ISA devices:\n");
 printf("--------------------------------------------------------------\n");
 while (dev)
 {
  printf("%-20.20s port=0x%02X;",dev->name,dev->iobase);
  if (dev->nirqs)
   {
    printf("IRQ=");
    for (x=0;x<dev->nirqs;x++)
      printf("%u;",dev->irqs[x]);
   }
  p = dev->properties;
  while (p)
   {
    printf("%s=%s;",p->name,p->value);
    p = p->next;
   }
  printf("\n");
  dev = dev->next;
 }
}

int ask_for_probe(char *name,int port)
{
 char n[10];

 sprintf(n," @ 0x%X",port);
 writech(40,0,' ',40,0x20);
 writes(70-strlen(name),0,name,0x20);
 writes(70,0,n,0x20);
 return 1;
}