/* ne.c: NE2000 and NE1000 compatible Network cards finder

   Copyright (c) 1998 Martin Hinner <martin@tdp.cz>
   Based on BSD's i386 Lpt port probe Geoff Rehmet - Rhodes University - 14/2/94

   This software may be used and distributed according to the terms
   of the GNU Public License, incorporated herein by reference.
 */
#include <conio.h>
#include <stdio.h>
#include <dos.h>

#include "dev.h"
#include "lpt.h"
#include "irqprobe.h"

/*
 * Internal routine to lptprobe to do port tests of one byte value
 */
static int lpt_port_test (int port, unsigned char data, unsigned char mask)
{
 int	temp, timeout;

 data = data & mask;
 outp(port, data);
 timeout = 10;
 do {
  delay(1);
  temp = inp(port) & mask;
  } while (temp != data && --timeout);
 return (temp == data);
}

int lpt_probe_irq;

void far lpt_probeirq (int irq)
{
 if (lpt_probe_irq==-1)
  lpt_probe_irq = irq;
}

 static unsigned char testbyte[18] = {
   0x55,			/* alternating zeros */
   0xaa,			/* alternating ones */
   0xfe, 0xfd, 0xfb, 0xf7,
   0xef, 0xdf, 0xbf, 0x7f,	/* walking zero */
   0x01, 0x02, 0x04, 0x08,
   0x10, 0x20, 0x40, 0x80	/* walking one */
  };

int lpt_probe(int port)
{
 int i;
 int irq;
 isadev *dev;

 outp(port+2, 0);
 /* Probe all bytes on port */
 for (i = 0; i < 18; i++)
  if (!lpt_port_test(port, testbyte[i], 0xff))
   return 0;

 /* write 0's to control and data ports */
 outp(port, 0);

 switch (port)
 {
  case 0x378: irq = 0x7; break;
  case 0x278: irq = 0x5; break;
  default   : irq = -1;
 }

 dev = alloc_isadev();
 setisadevname(dev,"PC Parallel port");
 setisadevport(dev,port);
 if (irq!=-1)
  allocisadevirq(dev,irq);
 add_isadevproperty(dev,"Mode",irq!=-1?"IRQ":"Polling");
 addisadev(dev);

 return 1;
}
