#include <stdio.h>
#include <dos.h>
#include <conio.h>

#include "dev.h"
#include "net/3c509.h"

int el3_current_tag = 0;

const char *if_names[] = {"10baseT", "AUI", "undefined", "BNC"};

#define EL3_DATA 0x00
#define EL3_CMD 0x0e
#define EL3_STATUS 0x0e
#define SelectWindow 1<<11

#define	 EEPROM_READ 0x80

int id_port;

#define EL3WINDOW(win_num) outpw(ioaddr + EL3_CMD,SelectWindow + (win_num))

/* Read a word from the EEPROM when in the ISA ID probe state. */
static unsigned short id_read_eeprom(int index)
{
  int bit, word = 0;

  /* Issue read command, and pause for at least 162 us. for it to complete.
     Assume extra-fast 16Mhz bus. */
  outp(id_port,EEPROM_READ + index);

  /* Pause for at least 162 us. for the read to take place. */
  delay (20);
	
  for (bit = 15; bit >= 0; bit--)
    word = (word << 1) + (inp(id_port) & 0x01);

   /*printf("  3c509 EEPROM word %d %#4.4x.\n", index, word);*/

  return word;
}

int el3_probe()
{
  short lrs_state = 0xff,i;
  unsigned char mac_addr[6];
  unsigned short iobase;
  int irq,if_port,ioaddr;
  int x;
  isadev *dev;

  /* Reset the ISA PnP mechanism on 3c509b. */
  outp(0x279,0x02);           /* Select PnP config control register. */
  outp(0xA79,0x02);           /* Return to WaitForKey state. */
  /* Select an open I/O location at 0x1*0 to do contention select. */

  for (id_port = 0x100; id_port < 0x200; id_port += 0x10) {
    outp(id_port,0x00);
    outp(id_port,0xff);
    if (inp(id_port) & 0x01)
	break;
  }
 if (id_port >= 0x200) {
	/* Rare -- do we really need a warning? */
	printf(" WARNING: No I/O port available for 3c509 activation.\n");
	return 0;
  }

  outp(id_port,0x00);
  outp(id_port,0x00);
  for(i = 0; i < 255; i++) {
     outp(id_port,lrs_state );
     lrs_state <<= 1;
     lrs_state = lrs_state & 0x100 ? lrs_state ^ 0xcf : lrs_state;
  }
  /* For the first probe, clear all board's tag registers. */
  if (el3_current_tag == 0)
	outp(id_port,0xd0);
  else				/* Otherwise kill off already-found boards. */
	outp(id_port,0xd8);

  if (id_read_eeprom(7) != 0x6d50) {
	return 0;
  }

  /* Read in EEPROM data, which does contention-select.
     Only the lowest address board will stay "on-line".
     3Com got the byte order backwards. */
  for (i = 0; i < 3; i++)
   {
    x = id_read_eeprom(i);
    mac_addr[i*2] = x>>8;
    mac_addr[(i*2)+1] = x&0xFF;
   }

  iobase=id_read_eeprom(8);
  if_port = iobase >> 14;
  ioaddr = 0x200 + ((iobase & 0x1f) << 4);
  irq = id_read_eeprom(9) >> 12;

  /* Set the adaptor tag so that the next card can be found. */
  outp(id_port,0xd0 + ++el3_current_tag);
  /* Activate the adaptor at the EEPROM location. */
  outp(id_port,(ioaddr >> 4) | 0xe0);
  EL3WINDOW(0);
  if (inpw(ioaddr) != 0x6d50)
	return 0;

  dev = alloc_isadev();
  setisadevname(dev,"3com 509 NIC");
  setisadevport(dev,ioaddr);
  allocisadevirq(dev,irq);
  add_isadevproperty(dev,"NIC Port",if_names[if_port]);
  add_isadevpropertyf(dev,"MAC Address","%02X:%02X:%02X:%02X:%02X:%02X",
	mac_addr[0],mac_addr[1],mac_addr[2],
	mac_addr[3],mac_addr[4],mac_addr[5]);
  add_isadevproperty(dev,"Configuration","Jumperless");
  addisadev(dev);

  return 1;
}

