/* ne.c: HP+  compatible Network cards finder

   Copyright (c) 1998 Martin Hinner <martin@tdp.cz>
   Portions (c) 1992-94 Donald Becker <becker@CESDIS.gsfc.nasa.gov>
   (from Linux kernel).

   This software may be used and distributed according to the terms
   of the GNU Public License, incorporated herein by reference.
 */
#include <conio.h>
#include <stdio.h>

#include "net/hp-plus.h"
#include "dev.h"


int hpplus_probe(int port)
{
  int i;
  unsigned char checksum;
  unsigned char mac_addr[6];
  unsigned int card_id;
  int irq;
  int options;
  int mem_start;

  if (!ask_for_probe("HP Plus NIC",port))
    return 0;

  /* Check for the HP+ signature, 50 48 0x 53. */
  if (inpw(port + HP_ID) != 0x4850 || (inpw(port + HP_PAGING) & 0xfff0) != 0x5300)
    return 0;

  /* Retrieve and checksum the station address. */
  outpw(port + HP_PAGING,HP_MAC_PAGE);
  checksum=0;
  for(i = 0; i < 6; i++)
   {
    mac_addr[i] = inp(port + 8 + i);
    checksum += mac_addr[i];
   }
  checksum += inp(port + 14);

  if (checksum != 0xff)
   {
    printf("hpplus_probe: Invalid checksum at 0x%x\n",port);
    return 0;
   }

  /* Point at the Software Configuration Flags. */
  outpw(port + HP_PAGING,HP_ID_PAGE);
  card_id = inpw(port + 12);

  outpw(port + HP_PAGING,HP_HW_PAGE);
  irq = inp(port + 13) & 0x0f;
  options = inpw(port + HPP_OPTION);
  if (options & HP_MEM_ENABLE)
     mem_start = inpw(port + 9) << 8;
     else
       mem_start = 0;

 printf("hpplus_probe: Found HP+ NIC at 0x%x, using IRQ %u, ID %04x, ",port,irq,card_id);
 if (mem_start)
  printf("Memory 0x%04X\n",mem_start);
   else printf("Programmed I/O\n");

 printf("hpplus_probe: MAC Address: ");
 for (i=0;i<6;i++)
  printf("%02X%c",mac_addr[i],i==5?'\n':':');

 return 1;
}
