/* ne.c: ICL EthetTeam 16i Network cards finder

   Copyright (c) 1998 Martin Hinner <martin@tdp.cz>
   Portions copyright (C) 1994-1998 by Mika Kuoppala

   This software may be used and distributed according to the terms
   of the GNU Public License, incorporated herein by reference.
 */
#include <stdio.h>
#include <conio.h>
#include "net/icl16.h"

#define slow_io() do { int _i = 40; while(--_i > 0) { inp(0x80); }}while(0)


static unsigned int icl16i_portlist[] =
   { 0x260, 0x280, 0x2A0, 0x240, 0x340, 0x320, 0x380, 0x300, 0 };
/* This is the Interrupt lookup table for Eth16i card */
static unsigned int icl16i_irqmap[] = { 9, 10, 5, 15, 0 };
/* This is the Interrupt lookup table for Eth32i card */
static unsigned int eth32i_irqmap[] = { 3, 5, 7, 9, 10, 11, 12, 15, 0 };  

char *icl16_porttype[] = {"BNC", "DIX", "TP", "AUTO", "FROM_EPROM" };

static void icl16i_eeprom_cmd(int ioaddr, unsigned char command)
{
	int i;

	outp(ioaddr + EEPROM_CTRL_REG, CS_0 | SK_0);
	outp(ioaddr + EEPROM_DATA_REG, DI_0);
	outp(ioaddr + EEPROM_CTRL_REG, CS_1 | SK_0);
	outp(ioaddr + EEPROM_DATA_REG, DI_1);
	outp(ioaddr + EEPROM_CTRL_REG, CS_1 | SK_1);

	for(i = 7; i >= 0; i--) {
		short cmd = ( (command & (1 << i)) ? DI_1 : DI_0 );
		outp(ioaddr + EEPROM_DATA_REG,cmd);
		outp(ioaddr + EEPROM_CTRL_REG,CS_1 | SK_0);
		slow_io();
		outp(ioaddr + EEPROM_CTRL_REG,CS_1 | SK_1);
		slow_io();
	}
}

static int icl16i_read_eeprom_word(int ioaddr)
{
	int i;
	int data = 0;

	for(i = 16; i > 0; i--) {
		outp(ioaddr + EEPROM_CTRL_REG,CS_1 | SK_0);
		slow_io();
		outp(ioaddr + EEPROM_CTRL_REG,CS_1 | SK_1);
		slow_io();
		data = (data << 1) | 
			((inp(ioaddr + EEPROM_DATA_REG) & DI_1) ? 1 : 0);
		
		slow_io();
	}

	return(data);
}

static int icl16i_read_eeprom(int ioaddr, int offset)
{
	int data = 0;

	icl16i_eeprom_cmd(ioaddr, EEPROM_READ | offset);
	outp(ioaddr + EEPROM_CTRL_REG,CS_1);
	data = icl16i_read_eeprom_word(ioaddr);
	outp(ioaddr + EEPROM_CTRL_REG,CS_0 | SK_0);
	return(data);
}

int icl16_probe(int port)
{
 int i;
 unsigned char creg[4] = { 0 };
 int irq;
 int ifport;
 unsigned short mac_addr[3];

 if(port < 0x1000) {
   if(icl16i_portlist[(inp(port + JUMPERLESS_CONFIG) & 0x07)]!= port)
     return 0;
  }

 for(i = 0; i < 4 ; i++) {
	creg[i] = inp(port + TRANSMIT_MODE_REG + i);
 }
 creg[0] &= 0x0F;      /* Mask collision cnr */
 creg[2] &= 0x7F;      /* Mask DCLEN bit */
 if( !((creg[2] == 0x36) && (creg[3] == 0xE0)) ) {
	creg[2] &= 0x40;
	creg[3] &= 0x03;
	if( !((creg[2] == 0x40) && (creg[3] == 0x00)) )
		return 0;
	}

 if(icl16i_read_eeprom(port, E_NODEID_0) != 0)
	return 0;

 if((icl16i_read_eeprom(port, E_NODEID_1) & 0xFF00) != 0x4B00)
	return 0;


 mac_addr[0] = icl16i_read_eeprom(port, E_NODEID_0);
 mac_addr[1] = icl16i_read_eeprom(port, E_NODEID_1);
 mac_addr[2] = icl16i_read_eeprom(port, E_NODEID_2);

 if( port < 0x1000) {
	irq = icl16i_irqmap[ ((inp(port + JUMPERLESS_CONFIG) & 0xC0) >> 6) ] ;
 } else
 {  /* Oh..the card is EISA so method getting IRQ different */
  int cbyte;
  unsigned short index = 0;
  cbyte = inp(port + EISA_IRQ_REG);
  while( (cbyte & 0x01) == 0)
   {
    cbyte = cbyte >> 1;
    index++;
   }
   irq =  eth32i_irqmap[ index ] ;
  }

 ifport = icl16i_read_eeprom(port, E_PORT_SELECT);
 printf("icl16_probe: Found ICL EtherTeam %s at 0x%X, using IRQ %u, Interface %s\n",
   port>0x1000?"32":"16i",port,irq,ifport<=5?icl16_porttype[ifport]:"Unknown");
 printf("icl16_probe: Mac address: ");
 for (i=0;i<3;i++)
   printf("%02X:%02X%c",mac_addr[i]>>8,mac_addr[i]&0xFF,i==2?'\n':':');

 return 1;
}