/* ne.c: NE2000 and NE1000 compatible Network cards finder

   Copyright (c) 1998 Martin Hinner <martin@tdp.cz>
   Portions (c) 1992-94 Donald Becker <becker@CESDIS.gsfc.nasa.gov>
   (from Linux kernel).

   This software may be used and distributed according to the terms
   of the GNU Public License, incorporated herein by reference.
 */

#include <conio.h>
#include <dos.h>
#include <stdio.h>

#include "net/ne.h"
#include "dev.h"

#include <stdio.h>
#include <dos.h>
#include <conio.h>

#include "irqprobe.h"


int ne_probe_port;
int ne_probe_irq;

void far
ne_probeirq (int irq)
{
  ne_probe_irq = irq;
}

int
ne_probe (int ioaddr)
{
 isadev *dev;

  struct
  {
    unsigned char value, offset;
  }
  program_seq[] =
  {
    {
      E8390_NODMA + E8390_PAGE0 + E8390_STOP, 0
    }
    ,				/* Select page 0 */
    {
      0x48, EN0_DCFG
    }
    ,				/* Set byte-wide (0x48) access. */
    {
      0x00, EN0_RCNTLO
    }
    ,				/* Clear the count regs. */
    {
      0x00, EN0_RCNTHI
    }
    ,
    {
      0x00, EN0_IMR
    }
    ,				/* Mask completion irq. */
    {
      0xFF, EN0_ISR
    }
    ,
    {
      E8390_RXOFF, EN0_RXCR
    }
    ,				/* 0x20  Set to monitor */
    {
      E8390_TXOFF, EN0_TXCR
    }
    ,				/* 0x02  and loopback mode. */
    {
      32, EN0_RCNTLO
    }
    ,
    {
      0x00, EN0_RCNTHI
    }
    ,
    {
      0x00, EN0_RSARLO
    }
    ,				/* DMA starting at 0x0000. */
    {
      0x00, EN0_RSARHI
    }
    ,
    {
      E8390_RREAD + E8390_START, 0
    }
  };

  /* A list of bad clones that we none-the-less recognize. */
  static struct
    {
      const char *name8, *name16;
      unsigned char SAprefix[4];
    }
  ne_bad_clone_list[] =
  {
    {
      "DE100", "DE200",
      {
	0x00, 0xDE, 0x01,
      }
    }
    ,
    {
      "DE120", "DE220",
      {
	0x00, 0x80, 0xc8,
      }
    }
    ,
    {
      "DFI1000", "DFI2000",
      {
	'D', 'F', 'I',
      }
    }
    ,				/* Original, eh?  */
    {
      "EtherNext UTP8", "EtherNext UTP16",
      {
	0x00, 0x00, 0x79
      }
    }
    ,
    {
      "NE1000", "NE2000-invalid",
      {
	0x00, 0x00, 0xd8
      }
    }
    ,				/* Ancient real NE1000. */
    {
      "NN1000", "NN2000",
      {
	0x08, 0x03, 0x08
      }
    }
    ,				/* Outlaw no-name clone. */
    {
      "4-DIM8", "4-DIM16",
      {
	0x00, 0x00, 0x4d,
      }
    }
    ,				/* Outlaw 4-Dimension cards. */
    {
      "Con-Intl_8", "Con-Intl_16",
      {
	0x00, 0x00, 0x24
      }
    }
    ,				/* Connect Int'nl */
    {
      "ET-100", "ET-200",
      {
	0x00, 0x45, 0x54
      }
    }
    ,				/* YANG and YA clone */
    {
      "COMPEX", "COMPEX16",
      {
	0x00, 0x80, 0x48
      }
    }
    ,				/* Broken ISA Compex cards */
    {
      "E-LAN100", "E-LAN200",
      {
	0x00, 0x00, 0x5d
      }
    }
    ,				/* Broken ne1000 clones */
    {
      "Ctron-8", "Ctron-16",
      {
	0x00, 0x00, 0x1d
      }
    }
    ,				/* ctron */
  };

  unsigned char SA_prom[32];
  int reg0;
  int regd;
  int i;
  int wordlength = 2;

  char *name = 0;

  if (!ask_for_probe("NEx000",ioaddr))
   return 0;

  reg0 = inp (ioaddr);
  if (reg0 == 0xFF)
    return 0;

  outp (ioaddr, E8390_NODMA + E8390_PAGE1 + E8390_STOP);
  regd = inp (ioaddr + 0x0d);
  outp (ioaddr + 0x0d, 0xff);
  outp (ioaddr, E8390_NODMA + E8390_PAGE0);
  inp (ioaddr + EN0_COUNTER0);	/* Clear the counter by reading. */
  if (inp (ioaddr + EN0_COUNTER0) != 0)
    {				/* Restore the old values. */
      outp (ioaddr, reg0);
      outp (ioaddr + 0x0d, regd);
      return 0;
    }

  outp (ioaddr + NE_RESET, inp (ioaddr + NE_RESET));	/* Reset NE card */
  i = 0;
  while ((inp (ioaddr + EN0_ISR) & ENISR_RESET) == 0)
    {
      delay (1);
      i++;
      if (i > 200)
	return 0;
    }
  outp (ioaddr + EN0_ISR, 0xff);	/* Ack all intr. */

  /* Reprogram NE card */
  for (i = 0; i < sizeof (program_seq) / sizeof (program_seq[0]); i++)
    outp (ioaddr + program_seq[i].offset, program_seq[i].value);

  for (i = 0; i < 32 /*sizeof(SA_prom) */ ; i += 2)
    {
      SA_prom[i] = inp (ioaddr + NE_DATAPORT);
      SA_prom[i + 1] = inp (ioaddr + NE_DATAPORT);
      if (SA_prom[i] != SA_prom[i + 1])
	wordlength = 1;
    }

  if (wordlength == 2)
    {
      for (i = 0; i < 16; i++)
	SA_prom[i] = SA_prom[i + i];
      outp (0x49, ioaddr + EN0_DCFG);	/* We must set the 8390 for word mode. */
    }

  ne_probe_irq = -1;
  init_irqprobe (ne_probeirq);
  outp (ioaddr + EN0_IMR, 0x50);	/* Enable one interrupt. */
  outp (ioaddr + EN0_RCNTLO, 0x00);
  outp (ioaddr + EN0_RCNTHI, 0x00);
  outp (ioaddr, E8390_RREAD + E8390_START);	/* Trigger it... */
  outp (ioaddr + EN0_IMR, 0x00);	/* Mask it again. */
  done_irqprobe ();

  /* Test for original NE2000 or NE1000 cards */
  if (SA_prom[14] == 0x57 && SA_prom[15] == 0x57)
    if (wordlength == 2)
      name = "NE2000";
    else
      name = "NE1000";

  for (i = 0; i < sizeof (ne_bad_clone_list) / sizeof (ne_bad_clone_list[0]); i++)
    {
      if (SA_prom[0] == ne_bad_clone_list[i].SAprefix[0] &&
	  SA_prom[1] == ne_bad_clone_list[i].SAprefix[1] &&
	  SA_prom[2] == ne_bad_clone_list[i].SAprefix[2])
	{
	  if (wordlength == 2)
	    name = ne_bad_clone_list[i].name16;
	  else
	    name = ne_bad_clone_list[i].name8;
	}
    }

  if (!name)
    if (wordlength == 2)
      name = "unknown NE2000 compatible card";
    else
      name = "unknown NE1000 compatible card";

  dev = alloc_isadev();
  setisadevname(dev,name);
  setisadevport(dev,ioaddr);
  allocisadevirq(dev,ne_probe_irq);
  add_isadevpropertyf(dev,"MAC Address","%02X:%02X:%02X:%02X:%02X:%02X",
	SA_prom[0],SA_prom[1],SA_prom[2],
	SA_prom[3],SA_prom[4],SA_prom[5]);
  addisadev(dev);

  return 1;
}
