/* smc.c: SMC EtherEZ ethernet cards finder

   Copyright (c) 1998 Martin Hinner <martin@tdp.cz>
   Portions (c) 1992-94 Donald Becker <becker@CESDIS.gsfc.nasa.gov>
   (from Linux kernel).

   This software may be used and distributed according to the terms
   of the GNU Public License, incorporated herein by reference.
 */

#include <conio.h>
#include <stdio.h>

#include "net/smc.h"
#include "dev.h"

/* Possible ioaddr values are: 200, 220, 240, 260, 280, 300, 340, 380 */
int 
smc_probe (int ioaddr)
{
  unsigned char idreg;
  unsigned char reg4;
  int checksum = 0;
  int piomode, addr, irqreg, i;
  unsigned char irqmap[8] =
  {0, 9, 3, 5, 7, 10, 11, 15};
  long addr_tbl[4] =
  {0x0C000, 0x0E000, 0xFC000, 0xFE000};
  int num_pages_tbl[4] =
  {0x20, 0x40, 0x80, 0xff};
  unsigned char mac_address[6];
  int irq, mem_start, num_pages;

  if (!ask_for_probe("SMC EtherEZ",ioaddr))
    return 0;

  idreg = inp (ioaddr + 7);
  reg4 = inp (ioaddr + 4) & 0x7f;

  /* Check the ID nibble */
  if ((idreg & 0xF0) != 0x20 && (idreg & 0xF0) != 0x40)
    return 0;

  /* Select the station address register set. */
  outp (ioaddr + 4, reg4);

  /* Test checksum */
  for (i = 0; i < 8; i++)
    checksum += inp (ioaddr + 8 + i);
  if ((checksum & 0xff) != 0xFF)
    return 0;

  /* Read MAC address */
  for (i = 0; i < 6; i++)
    mac_address[i] = inp (ioaddr + 8 + i);

  /* Switch from the station address to the alternate register set and read */
  /*  the useful registers there. */
  outp (ioaddr + 4, reg4 | 0x80);

  /* Enabled FINE16 mode to avoid BIOS ROM width mismatches at reboot. */
  outp (ioaddr + 0x0c, inp (ioaddr + 0x0c) | 0x80);

  piomode = inp (ioaddr + 0x8);
  addr = inp (ioaddr + 0xb);
  irqreg = inp (ioaddr + 0xd);

  /* Switch back to the station address register set so that the MS-DOS driver
     /* can find the card after a warm boot. */
  outp (ioaddr + 4, reg4);

  irq = irqmap[((irqreg & 0x40) >> 4) + ((irqreg & 0x0c) >> 2)];
  mem_start = ((addr & 0x0f) << 9) + addr_tbl[(addr >> 6) & 3];
  num_pages = num_pages_tbl[(addr >> 4) & 3];


  printf ("smc_probe: %s found at 0x%x, using IRQ %u %s\n",
	  (idreg & 0xF0) == 0x20 ? "SMC Ultra" : "SMC EtherEZ",
	  ioaddr,
	  irq,
	  piomode ? "(pio mode)" : "");

  printf ("smc_probe: Memory: %u pages at 0x%04X\n",
	  num_pages,
	  mem_start);

  printf ("smc_probe: MAC Address: ");
  for (i = 0; i < 6; i++)
    printf (" %2.2x", mac_address[i]);

  printf ("\n");
  return 1;
}


