/* wd.c: WD8003 and WD8013 compatible Network cards finder

   Copyright (c) 1998 Martin Hinner <martin@tdp.cz>
   Portions (c) 1993-94 Donald Becker <becker@CESDIS.gsfc.nasa.gov>
   (from Linux kernel).

   This software may be used and distributed according to the terms
   of the GNU Public License, incorporated herein by reference.
 */
#include <stdio.h>
#include <conio.h>

#include "dev.h"
#include "net/wd.h"
#include "irqprobe.h"

int wd_irqmap[] = {9,3,5,7,10,11,15,4};

int wd_probe_port;
int wd_probe_irq;

void far
wd_probeirq (int irq)
{
  wd_probe_irq = irq;
}

int wd_probe(int port)
{
 int i;
 int checksum;
 int mac_address[6];
 int mem_start = -1;
 char *model_name;
 int ancient = 0;

 if (!ask_for_probe("WD80x3 NIC",port))
   return 0;

 wd_probe_irq = -1;
 checksum = 0;
 for (i = 0; i < 8; i++)
   checksum += inp(port + 8 + i);
 if (inp(port + 8) == 0xff 	/* Extra check to avoid soundcard. */
     || inp(port + 9) == 0xff
     || (checksum & 0xff) != 0xFF)
    return 0;

 for (i = 0; i < 6; i++)
  mac_address[i] = inp(port + 8 + i);

 /* The following PureData probe code was contributed by
    Mike Jagdis <jaggy@purplet.demon.co.uk>. Puredata does software
    configuration differently from others so we have to check for them.
    This detects an 8 bit, 16 bit or dumb (Toshiba, jumpered) card.
 */

  if (inp(port+0) == 'P' && inp(port+1) == 'D')
   {
    unsigned char reg5 = inp(port+5);

    switch (inp(port+2))
     {
       case 0x03:  model_name = "PDI8023-8";	break;
       case 0x05: model_name = "PDUC8023";	break;
       case 0x0a: model_name = "PDI8023-16"; break;
       default:	 model_name = "PDI8023";break;
		/* Either 0x01 (dumb) or they've released a new version. */
     }
    mem_start = ((reg5 & 0x1c) + 0xc0) << 12;
    wd_probe_irq = (reg5 & 0xe0) == 0xe0 ? 10 : (reg5 >> 5) + 1;
   } else
   {
    /* This method of checking for a 16-bit board is borrowed from the
       we.c driver.  A simpler method is just to look in ASIC reg. 0x03.
       I'm comparing the two method in alpha test to make certain they
       return the same result. */

    /* Check for the old 8 bit board - it has register 0/8 aliasing.
       Do NOT check i>=6 here -- it hangs the old 8003 boards! */
    for (i = 0; i < 6; i++)
     if (inp(port+i) != inp(port+8+i))
       break;
    if (i >= 6)
     {
      model_name = "WD8003-old";
      mem_start = 0xD000;
      ancient = 1;
     } else
     {
      int tmp = inp(port+1); /* fiddle with 16bit bit */

      outp(port+1, tmp ^ 0x01); /* attempt to clear 16bit bit */
      if (((inp( port+1) & 0x01) == 0x01) /* A 16 bit card */
	  && (tmp & 0x01) == 0x01)        /* In a 16 slot. */
	model_name = "WD8013";
       else
	  model_name = "WD8003";
      outp(port+1,tmp);			/* Restore original reg1 value. */
     }
    }
 /* Read memory location ***************************************************/
 if (mem_start==-1)
 {
   int high_addr_bits = inp(port+5) & 0x1f;

   /* Some boards don't have the register 5 -- it returns 0xff. */
   if (inp(port+5)==0xff)
      high_addr_bits = 0x01;
   mem_start = ((inp(port)&0x3f) << 13) + (high_addr_bits << 19);
 }
 /* Probe IRQ **************************************************************/
 if (wd_probe_irq==-1)
  {
   if (ancient || inp(port+1) == 0xff)
    { /* We must probe IRQ, no way to read it... */
     outp(port+0x10,E8390_NODMA + E8390_STOP);
     outp(port+0x10+EN0_IMR,0x00);            /* Disable all intrs. */
     init_irqprobe (wd_probeirq);
     outp(port+0x10 + EN0_IMR,0xff);	/* Enable all interrupts. */
     outp(port+0x10 + EN0_RCNTLO,0x00);
     outp(port+0x10 + EN0_RCNTHI,0x00);
     outp(port+0x10,E8390_RREAD+E8390_START); /* Trigger it... */
     done_irqprobe ();
     outp(port+0x10+EN0_IMR,0x00);	/* Mask all intrs. again. */
    } else
     wd_probe_irq = wd_irqmap[((inp(port+4) >> 5) & 0x03) + (inp(port+1) & 0x04)];
  }
 if (wd_probe_irq == 2)	 /* Fixup bogosity: IRQ2 is really IRQ9 */
   wd_probe_irq = 9;

 /* Print summary *********************************************************/
 printf("wd_probe: %s at 0x%x, mem=0x%04x,",model_name,port,mem_start);
 if (wd_probe_irq==-1)
  printf("IRQ not detected\n");
   else printf(" IRQ %u\n",wd_probe_irq);
 printf("wd_probe: MAC address ");
 for (i = 0; i < 6; i++)
  printf("%2.2X%c",mac_address[i],i==5?'\n':':');
 return 1;
}
