/* serial.c: PC serial ports finder

   Copyright (c) 1998 Martin Hinner <martin@tdp.cz>
   Copyright (C) 1991, 1992  Linus Torvalds
     (from Linux Kernel)

   This software may be used and distributed according to the terms
   of the GNU Public License, incorporated herein by reference.
*/

#include <stdio.h>
#include <conio.h>
#include <dos.h>

#include "dev.h"
#include "serial.h"
#include "irqprobe.h"
#include "misc.h"

int serial_probe_irq;
int serial_probe_port;

void far
serial_probeirq (int irq)
{
 /* Test Interrupt Identification register */
 if (serial_probe_irq == -1)
   if ((inp(serial_probe_port+UART_IIR)&0x1) == 0x01)
     serial_probe_irq = irq;
}

int
serial_probe (int port)
{
 int scratch, scratch2,scratch3;
 int async_ok,type;
 int save_mcr,save_ier;
 char *str;
 int fifo;
 isadev *dev;

 if (!ask_for_probe("RS232 Serial port",port))
   return 0;

 scratch = inp(port + UART_IER);
 outp(port + UART_IER, 0);
 outp(0x080,0xFF);
 scratch2 = inp(port + UART_IER);
 outp(port + UART_IER, scratch);
 if (scratch2)
  return 0;

 scratch2 = inp(port + UART_LCR);
 outp(port + UART_LCR, 0xBF); /* set up for StarTech test */
 outp(port + UART_EFR, 0);	/* EFR is the same as FCR */
 outp(port + UART_LCR, 0);
 outp(port + UART_FCR, UART_FCR_ENABLE_FIFO);
 type = inp(port + UART_IIR) >> 6;
 outp(port + UART_LCR, scratch2);

 if (type == 0)  /* 16450 */
 {
  scratch = inp(port + UART_SCR);
  outp(port + UART_SCR, 0xa5);
  scratch3 = inp(port + UART_SCR);
  outp(port + UART_SCR, 0x5a);
  scratch2 = inp(port + UART_SCR);
  outp(port + UART_SCR, scratch);
  if ((scratch3 != 0xa5) || (scratch2 != 0x5a))
    type = 4; /* PORT_8250 */
 }

 if (type == 1) /* UNKNOWN */
   return 0;

 fifo=0;
 switch (type)
 {
  case 0: str = "Serial port 16450"; break;
  case 2: str = "Serial port 16550"; break;
  case 3: str = "Serial port 16550A";fifo=16; break;
  case 4: str = "Serial port 8250"; break;
 }

 scratch = inp(port + UART_MCR);
 outp(port + UART_MCR, UART_MCR_LOOP | scratch);
 outp(port + UART_MCR, UART_MCR_LOOP | 0x0A);
 scratch3 = inp(port + UART_MSR) & 0xF0;
 outp(port + UART_MCR, scratch);
 async_ok = (scratch3 == 0x90);

 /*
  * This routine detect the IRQ of a serial port by clearing OUT2 when
  * no UART interrupt are requested (IER = 0) (*GPL*). This seems to work at
  * each time, as long as no other device permanently request the IRQ.
  * If no IRQ is detected, or multiple IRQ appear, this function returns 0.
  * The variable "state" and the field "state->port" should not be null.
  */

 save_mcr = inp(port + UART_MCR);
 save_ier = inp(port + UART_IER);
 outp(port + UART_MCR, UART_MCR_OUT1 | UART_MCR_OUT2);

 serial_probe_irq = -1;
 serial_probe_port = port;
 init_irqprobe (serial_probeirq);

 outp(port + UART_MCR, 0);
 delay (1);
 outp(port + UART_MCR, UART_MCR_DTR | UART_MCR_RTS | UART_MCR_OUT2);
 outp(port + UART_IER, 0x0f);	/* enable all intrs */
 (void)inp(port + UART_LSR);
 (void)inp(port + UART_RX);
 (void)inp(port + UART_IIR);
 (void)inp(port + UART_MSR);
 outp(port + UART_TX, 0xFF);
 delay(2);

 done_irqprobe();

 outp(port + UART_MCR, save_mcr);
 outp(port + UART_IER, save_ier);

  dev = alloc_isadev();
  setisadevname(dev,str);
  setisadevport(dev,port);
  allocisadevirq(dev,serial_probe_irq);
  if (fifo)
   add_isadevpropertyf(dev,"FIFO","%u bytes",fifo);
  add_isadevproperty(dev,"Async test",async_ok?"passed":"failed");
  addisadev(dev);

 return 1;
}
